<?php
namespace CodingMs\Ftm\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Thomas Deuling <typo3@coding.ms>, coding.ms
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use CodingMs\Ftm\Utility\Console;
use CodingMs\Ftm\Utility\Log;
use CodingMs\Ftm\Utility\Tools;
use TYPO3\CMS\Extbase\Utility\ArrayUtility;
use \TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Controller for Theme-Overview and working on files
 *
 * @package ftm
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class OverviewController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * Theme file data service
	 *
	 * @var \CodingMs\Ftm\Service\ThemeFileDataService
	 * @inject
	 */
	protected $themeFileDataService;

	/**
	 * Theme service
	 *
	 * @var \CodingMs\Ftm\Service\ThemeService
	 * @inject
	 */
	protected $themeService;

	/**
	 * Storage service
	 *
	 * @var \CodingMs\Ftm\Service\StorageService
	 * @inject
	 */
	protected $storageService;

	/**
	 * Settings service
	 *
	 * @var \CodingMs\Ftm\Service\SettingsService
	 * @inject
	 */
	protected $settingsService;

	/**
	 * @var \TYPO3\CMS\Extbase\Object\ObjectManager
	 * @inject
	 */
	public $objectManager;

	/**
	 * @var array File data array
	 */
	protected $themeFileData = array();

	/**
	 * @var array Available Themes in the system
	 */
	protected $availableThemes = array();

	/**
	 * @var string Name of the selected theme
	 */
	protected $selectedTheme = '';

	/**
	 * @var array Available Theme-Scopes in the theme
	 */
	protected $themeScopes = array();

	/**
	 * @var string Name of the selected theme scope
	 */
	protected $selectedThemeScope = 'Theme';

	/**
	 * @var boolean A theme selected!?
	 */
	protected $themeSelected = FALSE;

	/**
	 * Initialize any action
	 */
	public function initializeAction() {

		// Create controller context for flash massages
		$this->controllerContext = $this->buildControllerContext();

		// Load and merge the settings
		$loadedSettings = $this->settingsService->getSettings();
		$this->settings = ArrayUtility::arrayMergeRecursiveOverrule($this->settings, $loadedSettings);

		// Validate the selected theme
		$this->validateSelectedTheme();
		$this->refreshThemeFiles();
	}

	/**
	 *
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @param \string $snippet
	 * @return void
	 * @since 2.0.0
	 */
	public function indexAction() {
		
		// meta.yaml parent-theme value

		// Set the settings! Required because of adjusting values!
		$this->view->assign('settings', $this->settings);
		$overviewColumnCount = $this->settingsService->getOverviewColumnCount($this->settings);
		$this->view->assign('overviewColSpan', ($overviewColumnCount+1));

		$this->view->assign('debugRequests', $this->request->getArguments());


		// Edit link for the Meta data
		$themeYaml = $this->themeService->getThemeYamlObject($this->selectedTheme);

		$editThemeYamlOnclick = 'window.location.href=\'alt_doc.php?returnUrl=\'+T3_THIS_LOCATION+\'&';
		$editThemeYamlOnclick .= 'edit[tx_ftm_domain_model_theme][' . $themeYaml->getUid() . ']=edit\'; return false;';
		if(Tools::getTypo3MajorVersion()>=7) {
			/** @var \TYPO3\CMS\Backend\Routing\UriBuilder $routingUriBuilder */
			$routingUriBuilder = $this->objectManager->get('TYPO3\\CMS\\Backend\\Routing\\UriBuilder');
			$editThemeYamlOnclick = $routingUriBuilder->buildUriFromRoute('record_edit', array('edit[tx_ftm_domain_model_theme][' . $themeYaml->getUid() . ']' => 'edit'));

			$editThemeYamlOnclick = 'window.location.href=\'' . $editThemeYamlOnclick . '&returnUrl=\'+T3_THIS_LOCATION; return false;';
			
		}
		$this->view->assign('editThemeYamlOnclick', $editThemeYamlOnclick);

		// Theme scope
		$this->view->assign('themeScopes', $this->themeScopes);
		$this->view->assign('selectedThemeScope', $this->selectedThemeScope);

		// File data
		$this->themeFileData = $this->themeFileDataService->getThemeFileData($this->selectedTheme, $this->selectedThemeScope, $this->settings['overview']['filter']);

		//die('inti'.$overviewColumnCount);
		$this->view->assign('themeFileData', $this->themeFileData);
		
		$this->view->assign('selectedTheme', $this->selectedTheme);
		$this->view->assign('availableThemes', $this->availableThemes);
		$this->view->assign('themeSelected', $this->themeSelected);
		
		// Readonly!?
		if($this->selectedTheme=='theme_bootstrap' || $this->selectedTheme=='theme_foundation') {
			$translation = LocalizationUtility::translate('tx_ftm_message.warning_theme_is_readonly', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING);
			$this->view->assign('themeReadonly', TRUE);
		}
		else {
			$this->view->assign('themeReadonly', FALSE);
		}
		
		// Update available?!
		global $TYPO3_CONF_VARS;
		$version = $TYPO3_CONF_VARS['EXTCONF']['ftm']['version'];
		$updateMessage = \CodingMs\Ftm\Service\UpdatesService::check('ftm', explode('.', $version));
		if($updateMessage) {
			$this->addFlashMessage($updateMessage, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING);
		}
		
	}

	/**
	 * Validates the selected theme
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @return void
	 * @since 2.0.0
	 */
	protected function validateSelectedTheme() {

		// Get all available themes
		$this->availableThemes = $this->themeService->getAvailableThemes();
		
		// Check if a theme is passed by request param
		if($this->request->hasArgument('theme')) {
			$validateTheme = $this->request->getArgument('theme');
			if($this->themeService->isValidTheme($validateTheme)) {
				$this->selectedTheme = $validateTheme;
			}
		}
		// try to read from settings
		else if(isset($this->settings['selectedTheme'])) {
			$validateTheme = $this->settings['selectedTheme'];
			if($this->themeService->isValidTheme($validateTheme)) {
				$this->selectedTheme = $validateTheme;
			}
		}
		
		// otherwise select the first available theme
		if($this->selectedTheme==='') {
			$this->selectedTheme = $this->themeService->getFirstAvailableTheme();
		}
		
		if($this->selectedTheme==='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
			$this->redirect('newTheme', 'Theme');
		}
		else {
			$this->settings['selectedTheme'] = $this->selectedTheme;
		}
		
		// Refresh theme yaml object
		// Compare the hash of the theme.yaml->array, with hash within DB
		$metaDataFromThemeYaml = $this->themeService->getMetaDataFromThemeYamlFile($this->selectedTheme);
		$changed = $this->themeService->loadThemeYaml($this->selectedTheme, $metaDataFromThemeYaml);
		
		if($changed=='yaml-object-refreshed') {
			$translation = LocalizationUtility::translate('tx_ftm_message.info_yaml_object_refreshed', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		}
		else if($changed=='yaml-file-refreshed') {
			$translation = LocalizationUtility::translate('tx_ftm_message.info_yaml_file_refreshed', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
		}
		$this->themeSelected = TRUE;

		// Validate theme scope
		$this->themeScopes = $this->themeService->getThemeScopes($this->selectedTheme);
		if($this->request->hasArgument('themeScope')) {
			$selectedThemeScopeForInArray = $this->request->getArgument('themeScope');
			if(array_key_exists($selectedThemeScopeForInArray, $this->themeScopes)) {
				$this->selectedThemeScope = $selectedThemeScopeForInArray;
			}
		}
		// try to read from settings
		else if(isset($this->settings['selectedThemeScope'])) {
			$selectedThemeScopeForInArray = $this->settings['selectedThemeScope'];
			if(array_key_exists($selectedThemeScopeForInArray, $this->themeScopes)) {
				$this->selectedThemeScope = $selectedThemeScopeForInArray;
			}
		}
		$this->settings['selectedThemeScope'] = $this->selectedThemeScope;

		// Validate the filter settings
		if(!empty($this->settings['overview']['filter'])) {
			foreach($this->settings['overview']['filter'] as $filter=>$value) {
				if($this->request->hasArgument('overview_filter_' . $filter)) {
					$this->settings['overview']['filter'][$filter] = $this->request->getArgument('overview_filter_' . $filter);
				}
			}
		}

		// Save settings data
		$this->settingsService->setSettings($this->settings);

		// Validate the file storage
		$storageUid = $this->storageService->storageExists($this->selectedTheme);
		if($storageUid===0) {
			$storageUid = $this->storageService->createStorage($this->selectedTheme);
			if($storageUid===0) {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_create_storage_failed', 'Ftm');
				$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.info_file_storage_created', 'Ftm');
				$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
			}
		}

	}

	/**
	 * @throws \Exception
	 */
	protected function refreshThemeFiles() {
		$themeYaml = $this->themeService->getMetaDataFromThemeYamlFile($this->selectedTheme);
		$parentTheme = $themeYaml['parentTheme']['name'];
		$extensions = $themeYaml['supportedExtensions'];
		// But don't work on theme_bootstrap
		if($this->selectedTheme!='theme_bootstrap' && $this->selectedTheme!='theme_foundation') {
			$filesSuccessful = $this->themeService->refreshThemeFiles($this->selectedTheme, $parentTheme, $extensions);
			if(!empty($filesSuccessful)) {
				$filesSuccessful = implode(', ', $filesSuccessful);
				$translation = LocalizationUtility::translate('tx_ftm_message.info_theme_typoscript_files_refreshed', 'Ftm', array($filesSuccessful));
				$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::INFO);
			}
		}
	}

}