<?php
namespace CodingMs\Ftm\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Thomas Deuling <typo3@coding.ms>, coding.ms
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use CodingMs\Ftm\Utility\Console;
use CodingMs\Ftm\Utility\Log;
use CodingMs\Ftm\Service\RealurlService;
use CodingMs\Ftm\Service\SettingsService;
use CodingMs\Ftm\Service\ThemeService;
use CodingMs\Ftm\Service\DocumentationService;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Extbase\Utility\ArrayUtility;
use TYPO3\CMS\Extbase\Utility\LocalizationUtility;
use CodingMs\Ftm\Service\ExtensionService;
use CodingMs\Ftm\Service\BackupService;

/**
 *
 *
 * @package ftm
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class ServicesController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * Settings service
	 *
	 * @var \CodingMs\Ftm\Service\SettingsService
	 * @inject
	 */
	protected $settingsService;

	/**
	 * Initialize any action
	 */
	public function initializeAction() {

		// Load and merge the settings
		$loadedSettings = $this->settingsService->getSettings();
		$this->settings = ArrayUtility::arrayMergeRecursiveOverrule($this->settings, $loadedSettings);

	}
	/**
	 *
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @return void
	 * @since 2.0.0
	 */
	public function indexAction() {

		$service = '';
		if($this->request->hasArgument('service')) {
			$service = $this->request->getArgument('service');
		}
		
		if($service=='realurlClearCache') {
			if(RealurlService::clearCache()) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_clearing_realurl_cache_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_clearing_realurl_cache_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}
		
		else if($service=='realurlDeleteAutoConfiguration') {
			if(RealurlService::deleteAutoConfiguration()) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_delete_realurl_auto_configuration_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_delete_realurl_auto_configuration_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}

		else if($service=='resetFtmSettings') {
			if(SettingsService::resetSettings()) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_reset_ftm_settings_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_reset_ftm_settings_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}

		else if($service=='fixPermissions') {
			if(ThemeService::fixPermissions($this->settings['selectedTheme'])) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_fix_file_permissions_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_fix_file_permissions_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}

		else if($service=='generateDocumentation') {
			if(DocumentationService::generateDocumentation($this->settings['selectedTheme'])) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_fix_file_permissions_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_fix_file_permissions_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}

		else if($service=='extensionsDeleteRepositoryCache') {
			if(ExtensionService::deleteRepositoryCache()) {
				$translation = LocalizationUtility::translate('tx_ftm_message.ok_delete_extension_repository_cache_successful', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::OK);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_delete_extension_repository_cache_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
		}

		else if($service=='systemBackupFullDatabase') {
			BackupService::backupFullDatabase();
		}

		

		// Set the settings! Required because of adjusting values!
		$this->view->assign('settings', $this->settings);
	}

}