<?php
namespace CodingMs\Ftm\Controller;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Thomas Deuling <typo3@coding.ms>, coding.ms
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use CodingMs\Ftm\Utility\Console;
use CodingMs\Ftm\Utility\Log;
use CodingMs\Ftm\Utility\Tools;
use TYPO3\CMS\Extbase\Utility\ArrayUtility;
use TYPO3\CMS\Core\Messaging\AbstractMessage;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Extbase\Utility\LocalizationUtility;

/**
 * Controller for creating, ... Themes
 *
 * @package ftm
 * @license http://www.gnu.org/licenses/gpl.html GNU General Public License, version 3 or later
 *
 */
class ThemeController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController {

	/**
	 * Theme service
	 *
	 * @var \CodingMs\Ftm\Service\ThemeService
	 * @inject
	 */
	protected $themeService;

	/**
	 * Settings service
	 *
	 * @var \CodingMs\Ftm\Service\SettingsService
	 * @inject
	 */
	protected $settingsService;

	/**
	 * Theme service
	 *
	 * @var \CodingMs\Ftm\Service\StorageService
	 * @inject
	 */
	protected $storageService;
	
	/**
	 * Initialize any action
	 */
	public function initializeAction() {

		// Load and merge the settings
	// DEV-DO_NOT_CACHE_SETTNGS
	//	$loadedSettings = $this->settingsService->getSettings();
	//	$this->settings = ArrayUtility::arrayMergeRecursiveOverrule($this->settings, $loadedSettings);
	}
	
	protected function validateSelectedTheme() {

		// Get all available themes
		$this->availableThemes = $this->themeService->getAvailableThemes();

		// Check if a theme is passed by request param
		$selectedTheme = '';
		if($this->request->hasArgument('theme')) {
			$validateTheme = $this->request->getArgument('theme');
			if($this->themeService->isValidTheme($validateTheme)) {
				$selectedTheme = $validateTheme;
			}
		}
		return $selectedTheme;
	}

	public function newFileAjaxAction() {

		// Default step is step 1
		$step = 1;
		
		// Message data
		$messages = array();
		$messages['form'] = array();
		$messages['error'] = array();
		$messages['success'] = array();
		$messages['request'] = $this->request->getArguments();
		$messages['settings'] = $this->request->getArguments();


		// Check the selected theme
		$selectedTheme = $this->validateSelectedTheme();
		if($selectedTheme=='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}

		// Theme scope
		$scopes = $this->themeService->getThemeScopes($selectedTheme);
		$selectedScope = 'Theme';
		$scopeFormSettings = 'Theme';
		if($this->request->hasArgument('scope')) {
			$selectedScopeForInArray = $this->request->getArgument('scope');
			if(array_key_exists($selectedScopeForInArray, $scopes) && $selectedScopeForInArray!='Theme') {
				$selectedScope = $selectedScopeForInArray;
				$scopeFormSettings = 'Extension';
			}
		}
		
		// File types
		$types = array();
		if(!empty($this->settings['files'][$scopeFormSettings])) {
			foreach($this->settings['files'][$scopeFormSettings] as $type=>$typeData) {
				$types[$type] = $type;
			}
			ksort($types);
		}

		// Selected file type
		$selectedType = '';
		if($this->request->hasArgument('type')) {
			$selectedTypeForInArray = $this->request->getArgument('type');
			if(array_key_exists($selectedTypeForInArray, $types)) {
				$selectedType = $selectedTypeForInArray;
				$step = 2;
			}
		}
		$messages['form']['type'] = $selectedType;


		// File sub types
		$subTypes = array();
		if (!empty($this->settings['files'][$scopeFormSettings][$selectedType])) {
			foreach ($this->settings['files'][$scopeFormSettings][$selectedType] as $subType => $subTypeData) {
				$subTypes[$subType] = $subType;
			}
			ksort($subTypes);
		}
		$messages['form']['subTypes'] = $subTypes;
		
		// When the file type is valid
		if($step==2) {

			// Selected file subType
			$selectedSubType = '';
			if ($this->request->hasArgument('subType')) {
				$selectedSubTypeForInArray = $this->request->getArgument('subType');
				if (array_key_exists($selectedSubTypeForInArray, $subTypes)) {
					$selectedSubType = $selectedSubTypeForInArray;
					$step = 3;
				}
			}
			$messages['form']['subType'] = $selectedSubType;


			// Directories
			$directories = array();
			$directoriesArray = array();
			if(isset($this->settings['files'][$scopeFormSettings][$selectedType])) {
				$directoriesArray = $this->settings['files'][$scopeFormSettings][$selectedType][$selectedSubType]['Directories'];
				if (!empty($directoriesArray)) {
					foreach ($directoriesArray as $directory => $directoryData) {
						$directories[$directory] = str_replace('_', '/', $directory);
					}
				}
				ksort($directories);
			}
			$messages['form']['directories'] = $directories;

			// When the directory is valid
			if($step==3) {
				// Directory
				$selectedDirectory = '';
				if ($this->request->hasArgument('directory')) {
					$selectedDirectoryForInArray = $this->request->getArgument('directory');
					if (array_key_exists($selectedDirectoryForInArray, $directoriesArray)) {
						$selectedDirectory = $selectedDirectoryForInArray;
						$step = 4;
					}
				}

				// When the directory is valid
				if($step==4) {

					// Load and add the snippets
					$snippetParams = array();
					$snippetParams['type'] = $selectedType;
					$snippetParams['subType'] = $selectedSubType;
					$snippetParams['scope'] = $selectedScope;
					$snippets = \CodingMs\Ftm\Service\SnippetService::searchSnippets($snippetParams, $this->settings);
					if ($snippets == NULL) {
						$translation = LocalizationUtility::translate('tx_ftm_message.error_snippet_loading_failed', 'Ftm');
						$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
						$snippets = array();
					}
					$messages['snippets']= $snippets;
					
					
					/*$this->view->assign('snippetParams', $snippetParams);
					$this->view->assign('snippets', $snippets);
					// Validate Snippet selection
					$snippetUid = 0;
					if ($this->request->hasArgument('snippetUid')) {
						$snippetUid = (int)$this->request->getArgument('snippetUid');
					}
					$this->view->assign('snippetUid', $snippetUid);*/

					// Validate the file extension
					// Get the first entry as default
					$extensions = $this->settings['files'][$scopeFormSettings][$selectedType][$selectedSubType]['Directories'][$selectedDirectory]['Extensions'];
					$messages['form']['extensions'] = $extensions;

				}
				
			}
		}

		$this->returnJson($messages);

	}

	/**
	 * PHP-Array to JSON conversion
	 *
	 * @param array $array
	 * @return void
	 */
	protected function returnJson(array $array=array()) {

		// Create JSON-String
		$json = json_encode($array);

		// Display data
		header("Content-Type: application/json; charset=utf-8");
		header("Content-Transfer-Encoding: 8bit");
		header("Content-Length: ".strlen($json));
		echo $json;
		exit;
	}

	
	public function newFileAction() {

		// Abort and back to overview
		if($this->request->hasArgument('abortAndBack')) {
			$this->redirect('index', 'Overview');
		}
		
		// Default step is step 1
		$step = 1;
		
		// Check the selected theme
		$selectedTheme = $this->validateSelectedTheme();
		if($selectedTheme=='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}
		
		// Theme scope
		$scopes = $this->themeService->getThemeScopes($selectedTheme);
		$selectedScope = 'Theme';
		$scopeFormSettings = 'Theme';
		if($this->request->hasArgument('scope')) {
			$selectedScopeForInArray = $this->request->getArgument('scope');
			if(array_key_exists($selectedScopeForInArray, $scopes) && $selectedScopeForInArray!='Theme') {
				$selectedScope = $selectedScopeForInArray;
				$scopeFormSettings = 'Extension';
			}
		}

		// File types
		$types = array();
		if(!empty($this->settings['files'][$scopeFormSettings])) {
			foreach($this->settings['files'][$scopeFormSettings] as $type=>$typeData) {
				$types[$type] = $type;
			}
			ksort($types);
		}
		// Selected file type
		$selectedType = '';
		if($this->request->hasArgument('type')) {
			$selectedTypeForInArray = $this->request->getArgument('type');
			if(array_key_exists($selectedTypeForInArray, $types)) {
				$selectedType = $selectedTypeForInArray;
				$step = 2;
			}
		}

		// When the file type is valid
		if($step==2) {


			// File sub types
			$subTypes = array();
			if(!empty($this->settings['files'][$scopeFormSettings][$selectedType])) {
				foreach($this->settings['files'][$scopeFormSettings][$selectedType] as $subType=>$subTypeData) {
					$subTypes[$subType] = $subType;
				}
				ksort($subTypes);
			}
			// Selected file subType
			$selectedSubType = '';
			if($this->request->hasArgument('subType')) {
				$selectedSubTypeForInArray = $this->request->getArgument('subType');
				if(array_key_exists($selectedSubTypeForInArray, $subTypes)) {
					$selectedSubType = $selectedSubTypeForInArray;
					$step = 3;
				}
			}


			// When the directory is valid
			if($step==3) {

				// Directories
				$directories = array();
				$directoriesArray = array();
				if(isset($this->settings['files'][$scopeFormSettings][$selectedType])) {
					$directoriesArray = $this->settings['files'][$scopeFormSettings][$selectedType][$selectedSubType]['Directories'];
					if (!empty($directoriesArray)) {
						foreach ($directoriesArray as $directory => $directoryData) {
							$directories[$directory] = str_replace('_', '/', $directory);
						}
					}
					ksort($directories);
				}
				// Directory
				$selectedDirectory = '';
				if($this->request->hasArgument('directory')) {
					$selectedDirectoryForInArray = $this->request->getArgument('directory');
					if(array_key_exists($selectedDirectoryForInArray, $directoriesArray)) {
						$selectedDirectory = $selectedDirectoryForInArray;
						$step = 4;
					}
				}


				// When the directory is valid
				if($step==4) {
					
					// Load and add the snippets
					$snippetParams = array();
					$snippetParams['type'] = $selectedType;
					$snippetParams['subType'] = $selectedSubType;
					$snippetParams['scope'] = $selectedScope;
					$snippets = \CodingMs\Ftm\Service\SnippetService::searchSnippets($snippetParams, $this->settings);
					if($snippets==NULL) {
						$translation = LocalizationUtility::translate('tx_ftm_message.error_snippet_loading_failed', 'Ftm');
						$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
						$snippets = array();
					}
					$this->view->assign('snippetParams', $snippetParams);
					$this->view->assign('snippets', $snippets);
					// Validate Snippet selection
					$snippetUid = 0;
					if ($this->request->hasArgument('snippetUid')) {
						$snippetUid = (int)$this->request->getArgument('snippetUid');
					}
					$this->view->assign('snippetUid', $snippetUid);

					// Validate the file extension
					// Get the first entry as default
					$extensions = $this->settings['files'][$scopeFormSettings][$selectedType][$selectedSubType]['Directories'][$selectedDirectory]['Extensions'];
					reset($extensions);
					$selectedExtension = key($extensions);
					// Check if there was passed a file extension
					if ($this->request->hasArgument('extension')) {
						$selectedExtensionForInArray = $this->request->getArgument('extension');
						if (array_key_exists($selectedExtensionForInArray, $extensions)) {
							$selectedExtension = $selectedExtensionForInArray;
						}
					}
					if (trim($selectedExtension) == '') {
						$translation = LocalizationUtility::translate('tx_ftm_message.error_please_select_a_valid_file_extension', 'Ftm');
						$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
					}

					// Validate the file name
					if ($this->request->hasArgument('fileName') && trim($selectedExtension) != '') {
						$fileName = $this->request->getArgument('fileName');
						if (Tools::checkValidFilename($fileName) && trim($fileName) != '') {

							// Load snippet content
							$content = '';
							if ($snippetUid > 0) {
								$snippetParams = array();
								$snippetParams['uid'] = $snippetUid;
								$snippet = \CodingMs\Ftm\Service\SnippetService::loadSnippet($snippetParams, $this->settings);
								if ($snippet['status'] == 'success' && isset($snippet['snippet']['content'])) {
									$content = $snippet['snippet']['content'];
								}
							}

							// Write the new file
							if($scopeFormSettings=='Extension') {
								$selectedDirectory = str_replace('Extensions_Extension', 'Extensions_' . $selectedScope, $selectedDirectory);
							}
							$directory = str_replace('_', '/', $selectedDirectory);
							$file = $fileName . '.' . $selectedExtension;
							$relPath = "typo3conf/ext/" . $selectedTheme . '/' . $directory . '/';
							if ($this->writeFile($relPath, $file, $content, TRUE)) {
								$step = 1;
								$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_file_successfully_created', 'Ftm');

								// Create and back to overview
								if($this->request->hasArgument('createAndBack')) {
									$this->redirect('index', 'Overview');
									$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
								}
								else if($this->request->hasArgument('createAndEdit')) {
/*
									http://business-website-leasing.de/typo3/mod.php?M=tools_FtmFtm&moduleToken=8e78531af8541c3f79e2fdf248ce610c5ee44e99&
									
									//tx_ftm_tools_ftmftm%5B	relPath%5D=Configuration%2FTypoScript%2F
									//&tx_ftm_tools_ftmftm%5B	name%5D=constants.txt
									//&tx_ftm_tools_ftmftm%5B	theme%5D=theme_bootstrap_nocheins
									//&tx_ftm_tools_ftmftm%5B	fileExtension%5D=txt
									
									//&tx_ftm_tools_ftmftm%5Baction%5D=editFile
									//
									//&tx_ftm_tools_ftmftm%5Bcontroller%5D=Theme
									*/
									$arguments = array();
									$arguments['name'] = $file;
									$arguments['relPath'] = $directory . '/';
									$arguments['theme'] = $selectedTheme;
									$arguments['fileExtension'] = $selectedExtension;
									$this->redirect('editFile', 'Theme', NULL, $arguments);
								}
								else {
									$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
								}
								
							}
							else {
								$translation = LocalizationUtility::translate('tx_ftm_message.error_theme_file_creation_failed', 'Ftm');
								$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
							}

						}
						else {
							$this->view->assign('fileName', $fileName);
							$translation = LocalizationUtility::translate('tx_ftm_message.error_please_enter_a_valid_file_name', 'Ftm');
							$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
						}
					}
					
					
					$this->view->assign('extensions', $extensions);
					$this->view->assign('selectedExtension', $selectedExtension);
					// End of step 4
				}

				$this->view->assign('directories', $directories);
				$this->view->assign('selectedDirectory', $selectedDirectory);
				// End of Step 3
			}

			$this->view->assign('subTypes', $subTypes);
			$this->view->assign('selectedSubType', $selectedSubType);
			// End of Step 2
		}

		$this->view->assign('step', $step);
		$this->view->assign('selectedTheme', $selectedTheme);
		$this->view->assign('selectedScope', $selectedScope);
		$this->view->assign('types', $types);
		$this->view->assign('selectedType', $selectedType);
	}

	public function deleteFileAction() {

		$success = FALSE;
		
		$selectedTheme = $this->validateSelectedTheme();
		if($selectedTheme=='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}

		/** @var \TYPO3\CMS\Core\Resource\ResourceStorage $storage */
		$storage = $this->storageService->getStorage($selectedTheme);
		if($storage!==NULL) {

			// Rel-Path is valid!?
			$relPath = '';
			if($this->request->hasArgument('relPath')) {
				$relPath = $this->request->getArgument('relPath');
			}
			
			// Name is valid!?
			$name = '';
			if($this->request->hasArgument('name')) {
				$name = $this->request->getArgument('name');
			}
			
			// Get file and delete
			$file = $storage->getFile($relPath . $name);
			if(\CodingMs\Ftm\Service\BackupService::backupFile(GeneralUtility::getFileAbsFileName($file->getPublicUrl()))) {
				$success = $storage->deleteFile($file);
			}
		}
		
		if($success) {
			$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_file_successfully_deleted', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		}
		else {
			$translation = LocalizationUtility::translate('tx_ftm_message.error_theme_file_deletion_fails', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
		}
		$this->redirect('index', 'Overview');
	}
	
	/**
	 * Writes a file
	 * 
	 * @param        $path
	 * @param        $file
	 * @param string $content
	 * @return bool
	 */
	protected function writeFile($path, $file, $content='', $createFolder=FALSE) {
		$absPath = GeneralUtility::getFileAbsFileName($path);

		// Create folder when it's not available
		if(!file_exists($absPath) && $createFolder) {
			GeneralUtility::mkdir_deep($absPath);
		}
		
		if(file_exists($absPath)) {
			if(!file_exists($absPath . $file)) {
				return GeneralUtility::writeFile($absPath . $file, $content, TRUE);
			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.warning_file_already_exists', 'Ftm');
				$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::WARNING);
			}
		}
		else {
			$translation = LocalizationUtility::translate('tx_ftm_message.error_directory_not_found', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
			return FALSE;
		}
		return FALSE;
	}

	/**
	 * Shows the create new Theme view
	 */
	public function newThemeAction() {

		$parentThemes = array();
		$parentThemes['none'] = LocalizationUtility::translate('tx_ftm_label.select_box_no_parent_theme', 'Ftm');
		$cloneThemes = array();
		$cloneThemes['none'] = LocalizationUtility::translate('tx_ftm_label.select_box_no_clone', 'Ftm');

		// Get all available themes
		$availableThemes = $this->themeService->getAvailableThemes();
		/** @var \KayStrobach\Themes\Domain\Model\Theme $theme */
		foreach($availableThemes as $theme) {
			$parentThemes[$theme->getExtensionName()] = $theme->getTitle() . ' (' . $theme->getExtensionName() . ')';
			$cloneThemes[$theme->getExtensionName()] = $theme->getTitle() . ' (' . $theme->getExtensionName() . ')';
		}
		
		// Selectboxes
		$this->view->assign('parentThemes', $parentThemes);
		$this->view->assign('cloneThemes', $cloneThemes);

		$parentTheme = '';
		// Set theme_bootstrap as default (if available)
		if(array_key_exists('theme_bootstrap', $parentThemes)) {
			$parentTheme = 'theme_bootstrap';
		}
		// Check if parent theme is passed by param
		if ($this->request->hasArgument('parentTheme')) {
			$parentTheme = $this->request->getArgument('parentTheme');
		}
		$this->view->assign('parentTheme', $parentTheme);
	}
	
	/**
	 * Neues Theme generieren
	 */
	public function createThemeAction() {

		$extensionKey = '';
		if ($this->request->hasArgument('extensionKey')) {
			$extensionKey = $this->request->getArgument('extensionKey');
		}
		$title = '';
		if ($this->request->hasArgument('title')) {
			$title = $this->request->getArgument('title');
		}
		$parentTheme = '';
		if ($this->request->hasArgument('parentTheme')) {
			$parentTheme = $this->request->getArgument('parentTheme');
		}
		$cloneTheme = '';
		if ($this->request->hasArgument('cloneTheme')) {
			$cloneTheme = $this->request->getArgument('cloneTheme');
		}
		$installTheme = '';
		if ($this->request->hasArgument('installTheme')) {
			$installTheme = $this->request->getArgument('installTheme')=='install' ? TRUE : FALSE;
		}


		$redirectAction = 'newTheme';
		$redirectController = 'Theme';
		$redirectArguments = $this->request->getArguments();
		try {

			// When the Theme was successfully created
			if($this->themeService->createNewTheme($extensionKey, $title, $parentTheme)) {

				// Clone files into the new Theme
				/**
				 * @todo theme cloning
				 */

				// Install the new Theme!?
				if($installTheme) {
					$this->themeService->installTheme($extensionKey);
					$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_created_and_installed', 'Ftm');
					$this->addFlashMessage($translation, '', AbstractMessage::OK);
					
					// Redirect actions
					$redirectAction = 'index';
					$redirectController = 'Overview';
					$redirectArguments = array('theme' => $extensionKey);
				}
				else {
					$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_created_and_not_installed', 'Ftm');
					$this->addFlashMessage($translation, '', AbstractMessage::OK);

					// Redirect actions
					$redirectAction = 'index';
					$redirectController = 'Overview';
					$redirectArguments = array();
				}

			}
			else {
				$translation = LocalizationUtility::translate('tx_ftm_message.error_theme_creation_failed', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
			
		} 
		
		/** @var \Exception $exception */
		catch (\Exception $exception) {
			$this->addFlashMessage($exception->getMessage(), '', AbstractMessage::ERROR);
		}

		// In case of errors
		// redirect back to new theme form
		// otherwise redirect to overview
		// If the theme-extensions was installed, select it directly	
		$this->redirect($redirectAction, $redirectController, 'Ftm', $redirectArguments);
	}
	
	public function saveSnippetAction() {

		$validParams = TRUE;
		
		$selectedTheme = $this->validateSelectedTheme();
		if($selectedTheme=='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}
		$this->view->assign('selectedTheme', $selectedTheme);

		$relPath = '';
		if($this->request->hasArgument('relPath')) {
			$relPath = $this->request->getArgument('relPath');
		}
		$this->view->assign('relPath', $relPath);

		$filename = '';
		if($this->request->hasArgument('filename')) {
			$filename = $this->request->getArgument('filename');
		}
		$this->view->assign('filename', $filename);

		/** @var \TYPO3\CMS\Core\Resource\ResourceStorage $storage */
		$content = '';
		if($this->request->hasArgument('content')) {
			$content = $this->request->getArgument('content');
		}
		$storage = $this->storageService->getStorage($selectedTheme);
		if($storage!==NULL && $content=='') {

			// Get file content
			$file = $storage->getFile($relPath . $filename);
			$content = $file->getContents();
		}
		$this->view->assign('content', $content);

		$title = '';
		if($this->request->hasArgument('title')) {
			$title = $this->request->getArgument('title');
		}
		$this->view->assign('title', $title);
		
		$type = '';
		if($this->request->hasArgument('type')) {
			$type = $this->request->getArgument('type');
		}
		$this->view->assign('type', $type);

		$subType = '';
		if($this->request->hasArgument('subType')) {
			$subType = $this->request->getArgument('subType');
		}
		$this->view->assign('subType', $subType);

		$scope = '';
		if($this->request->hasArgument('scope')) {
			$scope = $this->request->getArgument('scope');
		}
		$this->view->assign('scope', GeneralUtility::underscoredToUpperCamelCase($scope));

		$description = '';
		if($this->request->hasArgument('description')) {
			$description = $this->request->getArgument('description');
		}
		$this->view->assign('description', $description);

		$publicReadable = '';
		if($this->request->hasArgument('publicReadable')) {
			$publicReadable = $this->request->getArgument('publicReadable');
		}
		$this->view->assign('publicReadable', $publicReadable);

		$publicWritable = '';
		if($this->request->hasArgument('publicWritable')) {
			$publicWritable = $this->request->getArgument('publicWritable');
		}
		$this->view->assign('publicWritable', $publicWritable);
		
		
		

		if($this->request->hasArgument('saveAndBack')) {

			if(trim($title)=='') {
				$validParams = FALSE;
				$translation = LocalizationUtility::translate('tx_ftm_message.error_snippet_saving_failed_please_enter_a_title', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
			if(trim($description)=='') {
				$validParams = FALSE;
				$translation = LocalizationUtility::translate('tx_ftm_message.error_snippet_saving_failed_please_enter_a_description', 'Ftm');
				$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
			}
			
			if($validParams) {
				if (Tools::checkValidFilename($filename) && trim($filename) != '') {


					$snippetParams = array();
					$snippetParams['type'] = $type;
					$snippetParams['subType'] = $subType;
					$snippetParams['title'] = $title;
					$snippetParams['content'] = $content;
					$snippetParams['description'] = $description;
					$snippetParams['scope'] = $scope;
					$snippetParams['filename'] = $filename;
					$snippetParams['publicReadable'] = $publicReadable ? '1' : '0';
					$snippetParams['publicWritable'] = $publicWritable ? '1' : '0';


					$snippet = \CodingMs\Ftm\Service\SnippetService::saveSnippet($snippetParams, $this->settings);
					if ($snippet !== NULL) {
						/**
						 * @todo: Translate Messages from Service?!
						 */
						if ($snippet['status'] == 'success') {
							$this->addFlashMessage($snippet['message'], '', AbstractMessage::OK);
							$this->redirect('index', 'Overview');
						}
						else {
							$this->addFlashMessage($snippet['message'], '', AbstractMessage::ERROR);
						}
					}
					else {
						$translation = LocalizationUtility::translate('tx_ftm_message.error_snippet_saving_failed', 'Ftm');
						$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
					}

				}
				else {
					$translation = LocalizationUtility::translate('tx_ftm_message.error_please_enter_a_valid_file_name', 'Ftm');
					$this->addFlashMessage($translation, '', AbstractMessage::ERROR);
				}
			}
		}
		
		$this->view->assign('arguments', $this->request->getArguments());
	}

	public function editFileAction() {

		$success = FALSE;
		$relPath = '';
		$name = '';
		$fileContent = '';

		$selectedTheme = $this->validateSelectedTheme();
		if($selectedTheme=='') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}

		/** @var \TYPO3\CMS\Core\Resource\ResourceStorage $storage */
		$storage = $this->storageService->getStorage($selectedTheme);
		if($storage!==NULL) {

			// Rel-Path is valid!?
			$relPath = '';
			if($this->request->hasArgument('relPath')) {
				$relPath = $this->request->getArgument('relPath');
			}

			// Name is valid!?
			$name = '';
			if($this->request->hasArgument('name')) {
				$name = $this->request->getArgument('name');
			}

			// Get file and delete
			$file = $storage->getFile($relPath . $name);
			$fileContent = $file->getContents();
		}

		/*
		if($success) {
			$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_file_successfully_deleted', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		}
		else {
			$translation = LocalizationUtility::translate('tx_ftm_message.error_theme_file_deletion_fails', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
		}
		*/

		

		// Validate file extension
		$fileExtension = 'unknown';
		$scriptMode = 'ace/mode/text';
		$snippetFile = 'ace/snippets/text';
		$snippetScope = 'text';
		$snippets = '';

		if($this->request->hasArgument('fileExtension')) {
			$fileExtension = strtolower($this->request->getArgument('fileExtension'));
		}
		
		if($fileExtension=='js') {
			$scriptMode = 'ace/mode/javascript';
			$snippetFile = 'ace/snippets/javascript';
			$snippetScope = 'javascript';
		}
		else if($fileExtension=='css') {
			$scriptMode = 'ace/mode/css';
			$snippetFile = 'ace/snippets/css';
			$snippetScope = 'css';
		}
		else if($fileExtension=='less') {
			$scriptMode = 'ace/mode/less';
			$snippetFile = 'ace/snippets/less';
			$snippetScope = 'less';
			$snippets = $this->settings['editor']['snippets']['less'];
		}
		else if($fileExtension=='html') {
			$scriptMode = 'ace/mode/twig';
			$snippetFile = 'ace/snippets/twig';
			$snippetScope = 'twig';

			$snippets = "\n";
			$snippets .= "m.snippets.push({\n";
			$snippets .= "	content: '<f:debug>\${1:_all}</f:debug>',\n";
			$snippets .= "	name: '<f:debug>',\n";
			$snippets .= "	tabTrigger: '<f:debug'\n";
			$snippets .= "});";
			
		}

		// All JavaScript code which must be wrappen in require/jquery-ready
		$javaScript = \CodingMs\Ftm\Service\JavaScriptService::getAceInit($this->settings, $snippetFile, $snippetScope, $scriptMode);
		$javaScript .= \CodingMs\Ftm\Service\JavaScriptService::getAceAutoCompletion($snippets);
		$javaScript .= \CodingMs\Ftm\Service\JavaScriptService::getAceCommands();

		$libraries = array();
		$libraries[] = "ace/ace";
		$libraries[] = $scriptMode;
		$libraries[] = $snippetFile;
		$libraries[] = $this->settings['editor']['theme'];

		$libraries[] = 'ace/ext/language_tools';
		$libraries[] = 'ace/snippets';
		$libraries[] = 'ace/config';
		$libraries[] = 'ace/ext/language_tools';
		
		$javaScript = \CodingMs\Ftm\Service\JavaScriptService::getAceRequireWrap($javaScript, '../typo3conf/ext/ftm/Resources/Public/Contrib/Ace/ace', $libraries);
		$aceEditorLibrary = '';

		// Global JavaScript code
		$javaScript .= \CodingMs\Ftm\Service\JavaScriptService::getAceResize();

		$this->view->assign('javaScript', $javaScript);
		$this->view->assign('aceEditorLibrary', $aceEditorLibrary);
		
		
		
		$this->view->assign('fileContent', $fileContent);
		$this->view->assign('fileExtension', $fileExtension);
		$this->view->assign('scriptMode', $scriptMode);
		$this->view->assign('snippetFile', $snippetFile);
		$this->view->assign('snippetScope', $snippetScope);
		$this->view->assign('relPath', $relPath);
		$this->view->assign('fileName', $name);
		$this->view->assign('selectedTheme', $selectedTheme);
		$this->view->assign('settings', $this->settings);
	}

	public function saveFileAction() {

		$response = array();
		
		$selectedTheme = $this->validateSelectedTheme();
		if ($selectedTheme == '') {
			$translation = LocalizationUtility::translate('tx_ftm_exception.validating_theme_fails', 'Ftm');
			throw new \Exception($translation);
		}

		/** @var \TYPO3\CMS\Core\Resource\ResourceStorage $storage */
		$storage = $this->storageService->getStorage($selectedTheme);
		if ($storage instanceof \TYPO3\CMS\Core\Resource\ResourceStorage) {

			// Rel-Path is valid!?
			$relPath = '';
			if ($this->request->hasArgument('relPath')) {
				$relPath = $this->request->getArgument('relPath');
			}

			// Name is valid!?
			$name = '';
			if ($this->request->hasArgument('name')) {
				$name = $this->request->getArgument('name');
			}

			// Get file and delete
			$file = $storage->getFile($relPath . $name);
			if ($file instanceof \TYPO3\CMS\Core\Resource\File) {
				if ($this->request->hasArgument('content')) {
					
					/** @var \TYPO3\CMS\Extbase\Persistence\Generic\PersistenceManager $persistenceManager
					$persistenceManager = $this->objectManager->get('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
					$persistenceManager->persistAll(); 
					$response['content'] = $this->request->getArgument('content') . ' written!!';
					// --> doesn't work, because of our dirty exit

					$response['publicUrl'] = $file->getPublicUrl();
					$response['publicUrlFull'] = GeneralUtility::getFileAbsFileName($file->getPublicUrl());
					//$response['success'] = file_put_contents($publicUrl, $this->request->getArgument('content')) . ' Bytes written in file ' . $publicUrl;
					 */
					
					
					if(\CodingMs\Ftm\Service\BackupService::backupFile(GeneralUtility::getFileAbsFileName($file->getPublicUrl()))) {
						$file->setContents($this->request->getArgument('content'));
						$response['success'] = $file->getSize() . ' written!!';
					}
					else {
						$response['error'] = 'File backup failed, file not saved';
					}
					
				}
			}
			else {
				$response['error'] = 'File not found';
			}

		}
		else {
			$response['error'] = 'Storage not found';
		}

		/**
		 * @todo: add translations
		 */
		
		/*
		if($success) {
			$translation = LocalizationUtility::translate('tx_ftm_message.ok_theme_file_successfully_deleted', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::OK);
		}
		else {
			$translation = LocalizationUtility::translate('tx_ftm_message.error_theme_file_deletion_fails', 'Ftm');
			$this->addFlashMessage($translation, '', \TYPO3\CMS\Core\Messaging\AbstractMessage::ERROR);
		}
		*/

			
		return json_encode($response);
	}
	
}