<?php
namespace CodingMs\Ftm\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2012 Thomas Deuling <typo3@coding.ms>, coding.ms
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use \CodingMs\Ftm\Utility\Tools;
use \TYPO3\CMS\Core\Utility\GeneralUtility;

/**
 * Builds required JavaScript
 *
 * @package ftm
 * @subpackage Service
 * 
 * @author Thomas Deuling <typo3@coding.ms>
 * @since 2.0.0
 */
class JavaScriptService {

	
	public static function getAceInit(array $settings=array(), $snippetFile='', $snippetScope='', $scriptMode='') {
		$js = "
				var snippetFile = '" . $snippetFile . "';
				var snippetScope = '" . $snippetScope . "';
				
				// Set up the editor
				editor = ace.edit('editor');
				editor.setTheme('" . $settings['editor']['theme'] . "');
				editor.getSession().setMode('" . $scriptMode . "');
	
				// Using soft tabs? (that means spaces instead of a tab)
				editor.getSession().setTabSize(" . $settings['editor']['tabs']['tabSize'] . ");
				editor.getSession().setUseSoftTabs(" . $settings['editor']['tabs']['useSoftTabs'] . ");
		
				// Show invisible characters?!
				editor.setShowInvisibles(" . $settings['editor']['showInvisibles'] . ");
				
				
				// Resize editor
				//listen for changes
				TYPO3.jQuery(window).resize(resizeEditor);
				//set initially
				resizeEditor();
		";
		return $js;
	}
	
	public static function getAceRequireWrap($javaScript='', $path='', array $libraries=array()) {
		
		$libraries = "'" . implode("', '", $libraries) . "'";
		
		$js = "
			var editor;
			// Tell RequireJS where ace is located
			require.config({
				paths: {
					'ace': '" . $path . "'
				}
			});
				
			// Load the ace module
			require([" . $libraries . "], function(ace) {
				" . $javaScript . "
			});
		";
		return $js;
	}
	
	public static function getAceJqueryWrap($javaScript='') {
		$js = "
			var editor;
			jQuery(document).ready(function () {
				" . $javaScript . "
			});
		";
		return $js;
	} 
	
	/**
	 * @return string Command JavaScript
	 */
	public static function getAceCommands() {
		$js = "
				editor.commands.addCommand({
					name: 'save',
					bindKey: {
					win: 'Ctrl-S',
						mac: 'Command-S',
						sender: 'editor|cli'
					},
					exec: function() {
						saveFileAjax()
					}
				});
		";
		return $js;
	}

	/**
	 * @return string Command JavaScript
	 */
	public static function getAceResize() {
		$js = "
			// Resize editor
			function resizeEditor() {
				var jQuery = TYPO3.jQuery;
				return jQuery('#editor').height(jQuery(window).height() - 50);
			}
			function saveFileAjax() {
				var jQuery = TYPO3.jQuery;
				var content = editor.getSession().getValue();
				jQuery.post(uriSave, {
						'tx_ftm_tools_ftmftm[content]': content
					},
					function(response) {
						if(typeof(response.error)!='undefined') {
							alert(response.error);
							jQuery('.ace_content').css('background-color', '#660000');
							setTimeout(function() { jQuery('.ace_content').css('background-color', 'transparent') }, 300);
						}
						else {
							jQuery('.ace_content').css('background-color', '#006600');
							setTimeout(function() { jQuery('.ace_content').css('background-color', 'transparent') }, 300);
						}
					}
				);
			}
		";
		return $js;
	}

	public static function getAceAutoCompletion($snippets) {
		$js = "
		
				//Include auto complete- Only for Template Editor page
				ace.config.loadModule('ace/ext/language_tools', function () {
					editor.setOptions({
						enableBasicAutocompletion: true,
						enableSnippets: true
					});
		
					var snippetManager = ace.require('ace/snippets').snippetManager;
					var config = ace.require('ace/config');
		
					ace.config.loadModule(snippetFile, function(m) {
						if (m) {
		
							//console.log(m);
							//console.log(snippetManager.files, m.snippetText, m.snippets);
		
							snippetManager.files[snippetScope] = m;
							if(typeof(m.snippetText) != 'undefined') {
								m.snippets = snippetManager.parseSnippetFile(m.snippetText);
							}
							else {
								m.snippets = snippetManager.parseSnippetFile('');
							}
		
							//console.log(snippetManager.files);
		
							// or do this if you already have them parsed
							m.snippets.push({
								content: 'hello \${1:world}...!',
								name: 'hello',
								tabTrigger: 'h'
							});
		
							" . $snippets . "
		
							snippetManager.register(m.snippets, m.scope);
						}
					});
		
				});
		";
		
		return $js;
	}
	
}

?>