<?php
namespace CodingMs\Ftm\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) 2014 Thomas Deuling <typo3@coding.ms>, coding.ms
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use CodingMs\Ftm\Utility\Tools;
use \TYPO3\CMS\Core\Utility\GeneralUtility;
use \TYPO3\CMS\Backend\Utility\BackendUtility;

/**
 * Theme file data service
 *
 * @package ftm
 * @subpackage Service
 */
class ThemeFileDataService {

	/**
	 * @var array Theme file data
	 */
	protected $themeFileData = array();

	/**
	 * @var string Absolute theme path
	 */
	protected $absPath = '';

	/**
	 * @var string Theme name
	 */
	protected $theme = '';

	/**
	 * @var string Theme name
	 */
	protected $themeReadOnly = TRUE;

	/**
	 * @var string Theme scope
	 */
	protected $themeScope = '';

	/**
	 * @var array Filter
	 */
	protected $filter = array();

	/**
	 * Gets the file data of a theme
	 *
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @param \string $theme
	 * @param \array $filter
	 * @return array Array with theme file data
	 * @since 2.0.0
	 */
	public function getThemeFileData($theme, $themeScope, array $filter=array()) {
		$this->theme = $theme;
		if($theme!='theme_bootstrap' && $theme!='theme_foundation') {
			$this->themeReadOnly = FALSE;
		}
		$this->themeScope = $themeScope;
		$this->filter = $filter;
		$this->absPath = GeneralUtility::getFileAbsFileName('typo3conf/ext/' . $this->theme);
		if(file_exists($this->absPath)) {
			$this->iterateDir($this->absPath);
		}
		ksort($this->themeFileData);
		return $this->themeFileData;
	}

	/**
	 * Iterate
	 * 
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @param \string $path
	 * @return void
	 * @since 2.0.0
	 */
	protected function iterateDir($path) {
		/** @var \DirectoryIterator $file */
		foreach(new \DirectoryIterator($path) as $file) {
			if(!$file->isDot()) {
				if($file->isDir()) {
					$this->iterateDir($file->getPathname());
				}
				else {
					$this->extractFileData($file);
				}
			}
		}
	}

	/**
	 * Extracts file data
	 * 
	 * @author Thomas Deuling <typo3@coding.ms>
	 * @param \DirectoryIterator $file
	 * @return void
	 * @since 2.0.0
	 */
	protected function extractFileData(\DirectoryIterator $file) {

		$relPath = str_replace($this->absPath, '', $file->getPath());
		$relPathFilename = $relPath . '/' . $file->getFilename();
		// First index is empty!!
		$relPathParts = explode('/', $relPath);
		$fileData = array();




		// General
		$fileData['scope'] = 'Theme';
		$fileData['relPath'] = substr($relPath, 1, strlen($relPath)-1) . '/';
		$fileData['absPath'] = $file->getPath();
		$fileData['name'] = $file->getFilename();
		$fileData['extension'] = $file->getExtension();

		// Permissions
		$fileData['isWritable'] = $this->themeReadOnly ? FALSE : $file->isWritable();
		$fileData['isReadable'] = $file->isReadable();
		
		if($fileData['isReadable']) {
			$fileData['permissionsString'] = '<span style="color: green">Read</span> / ';
		}
		else {
			$fileData['permissionsString'] = '<span style="color: red">Read</span> / ';
		}
		if($fileData['isWritable']) {
			$fileData['permissionsString'] .= '<span style="color: green">Write</span>';
		}
		else {
			$fileData['permissionsString'] .= '<span style="color: red">Write</span>';
		}
		
		// Size
		$fileData['size'] = -1;
		if($fileData['isReadable']) {
			$fileData['size'] = $file->getSize();
		}
		$fileData['sizeString'] = Tools::convertFileSizeToString($fileData['size']);
		
		// Extension data
		$fileData['isExtension'] = FALSE;
		$fileData['extensionKey'] = ''; //GeneralUtility::underscoredToUpperCamelCase($this->theme);
		if($relPathParts[3] === 'Extensions') {
			$fileData['isExtension'] = TRUE;
			$fileData['extensionKey'] = GeneralUtility::underscoredToUpperCamelCase($relPathParts[4]);
			$fileData['scope'] = $fileData['extensionKey'];
		}
		
		// Check the scope
		// If scope:theme and file from scope:extension
		if($this->themeScope=='Theme' && $fileData['isExtension']) {
			return;
		}
		// If scope:extension and file from scope:theme
		if($this->themeScope!='Theme' && !$fileData['isExtension']) {
			return;
		}
		// If scope:extension but file from a different extension
		$themeScopeUpperCamelCase =  GeneralUtility::underscoredToUpperCamelCase($this->themeScope);
		if($fileData['isExtension'] && $themeScopeUpperCamelCase!= $fileData['extensionKey']) {
			return;
		}
		
		// No hidden folder/files, processed folder, etc.
		if($relPathParts[1]=='_processed_' || substr($relPathParts[1], 0, 1)=='.' || substr($fileData['name'], 0, 1)=='.') {
			return;
		}

			

		$fileData['fullFilePath'] = 'typo3conf/ext/theme_test_ftm' . $relPathFilename;


		//BackendUtility::getModuleUrl('tools_FtmFtm');
		//$returnUrl = '&returnUrl=' . rawurlencode(BackendUtility::getModuleUrl('tools_FtmFtm', array(), FALSE, TRUE));
		//$fileData['editFileLink'] = 'file_edit.php?target=typo3conf/ext/' . $this->theme . $relPathFilename . $returnUrl;



		$imageTypes = array('gif', 'jpg', 'jpeg', 'png', 'ico', 'svg');
		$mediaTypes = array('swf', 'flv', 'otf', 'ttf', 'woff', 'woff2', 'eot');
		$fontTypes = array('otf', 'ttf', 'woff', 'woff2', 'eot');
		$flashTypes = array('swf', 'flv');
		$themeTypes = array('txt', 'php', 'yaml', 'sql');
		$languageTypes = array('xml', 'xlf');


		$fileData['isEditable'] = FALSE;
		
		// Theme scope
		if($this->themeScope=='Theme') {

			// PageTS
			if($fileData['extension']=='pagets') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Page';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['name']=='tsconfig.txt') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Page';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template';
				$fileData['isEditable'] = FALSE;
			}

			// Constants & Setup
			else if($fileData['extension']=='constantsts') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Constants';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['extension']=='setupts') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Setup';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['name']=='setup.txt') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Setup';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template';
				$fileData['isEditable'] = FALSE;
			}
			else if($fileData['name']=='constants.txt') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Constants';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template';
				$fileData['isEditable'] = FALSE;
			}

			// User TypoScript
			else if($fileData['extension']=='userts') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'User';
				$fileData['icon'] = 't3-icon t3-icon-apps t3-icon-apps-pagetree t3-icon-pagetree-backend-user';
				$fileData['isEditable'] = TRUE;
			}
			
			// Documentation
			else if($fileData['extension']=='rst') {
				$fileData['type'] = 'Documentation';
				$fileData['subType'] = 'ReStructuredText';
				$fileData['icon'] = 't3-icon t3-icon-actions t3-icon-actions-system t3-icon-system-extension-documentation';
				$fileData['isEditable'] = TRUE;
			}
			
			// Fluid
			else if($fileData['extension']=='html') {
				$fileData['type'] = 'Fluid';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-html';
				$fileData['isEditable'] = TRUE;
				if($relPathParts[3] === 'Layouts') {
					$fileData['subType'] = $this->getSubType('Layouts', $relPathParts);
				}
				else if($relPathParts[3] === 'Partials') {
					$fileData['subType'] = $this->getSubType('Partials', $relPathParts);
				}
				else if($relPathParts[3] === 'Templates') {
					$fileData['subType'] = $this->getSubType('Templates', $relPathParts);
				}
				else if($relPathParts[5] === 'Widget') {
					$fileData['subType'] = $this->getSubType('Widget', $relPathParts);
				}
			}

			// JavaScript
			else if($fileData['extension']=='js') {
				$fileData['type'] = 'JavaScript';
				$fileData['subType'] = strtoupper($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-js';
				$fileData['isEditable'] = TRUE;
			}

			// Dyncss/CSS
			else if($fileData['extension']=='less') {
				$fileData['type'] = 'Dyncss';
				$fileData['subType'] = strtoupper($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['extension']=='sass') {
				$fileData['type'] = 'Dyncss';
				$fileData['subType'] = ucfirst($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['extension']=='css') {
				$fileData['type'] = 'Stylesheets';
				$fileData['subType'] = strtoupper($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}

			// Theme/System
			else if($fileData['extension']=='php') {
				$fileData['type'] = 'Theme';
				$fileData['subType'] = 'Theme';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-php';
			}
			else if($fileData['name']=='theme.yaml') {
				$fileData['type'] = 'Theme';
				$fileData['subType'] = 'Meta';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-other t3-icon-other-other';
			}
			else if($fileData['name']=='ext_icon.png' || $fileData['name']=='ext_icon.png') {
				$fileData['type'] = 'Theme';
				$fileData['subType'] = 'Image';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-other t3-icon-other-other';
			}
			/*else if(in_array($fileData['extension'], $themeTypes)) {
				$fileData['type'] = 'Theme';
				$fileData['subType'] = strtoupper($fileData['extension']);
			}*/

			// Images
			else if(in_array($fileData['extension'], $imageTypes)) {
				$fileData['type'] = 'Image';
				$fileData['subType'] = $fileData['relPath'];
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-media t3-icon-media-image';
			}
			
		}
		
		// File is an Extension file
		else if($this->themeScope!='Theme') {

			// PageTS
			if($fileData['name']=='tsconfig.txt') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Page';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}

			// Constants & Setup
			else if($fileData['name']=='setup.txt' || $fileData['extension']=='setupts') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Setup';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['name']=='constants.txt' || $fileData['extension']=='constantsts') {
				$fileData['type'] = 'TypoScript';
				$fileData['subType'] = 'Constants';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-x t3-icon-x-content-template-extension';
				$fileData['isEditable'] = TRUE;
			}

			// Dyncss/CSS
			else if($fileData['extension']=='less') {
				$fileData['type'] = 'Dyncss';
				$fileData['subType'] = ucfirst($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['extension']=='sass') {
				$fileData['type'] = 'Dyncss';
				$fileData['subType'] = ucfirst($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}
			else if($fileData['extension']=='css') {
				$fileData['type'] = 'Stylesheets';
				$fileData['subType'] = ucfirst($fileData['extension']);
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-css';
				$fileData['isEditable'] = TRUE;
			}

			// Fluid
			else if($fileData['extension']=='html') {
				$fileData['type'] = 'Fluid';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-html';
				$fileData['isEditable'] = TRUE;
				if($relPathParts[5] === 'Layouts') {
					$fileData['subType'] = $this->getSubType('Layouts', $relPathParts);
				}
				else if($relPathParts[5] === 'Partials') {
					$fileData['subType'] = $this->getSubType('Partials', $relPathParts);
				}
				else if($relPathParts[5] === 'Templates') {
					$fileData['subType'] = $this->getSubType('Templates', $relPathParts);
				}
				else if($relPathParts[7] === 'Widget') {
					$fileData['subType'] = $this->getSubType('Widget', $relPathParts);
				}
			}

			// Traditional-Template/Fluid
			else if($fileData['extension']=='tmpl') {
				$fileData['type'] = 'Fluid';
				$fileData['icon'] = 't3-icon t3-icon-mimetypes t3-icon-mimetypes-text t3-icon-text-html';
				$fileData['isEditable'] = TRUE;
				$fileData['subType'] = 'Templates';
			}




		}
		
		/*
		// PageTS
		if($fileData['extension']=='pagets') {
			$fileData['type'] = 'TypoScript';
			if($relPathParts[2] === 'Elements') {
				$fileData['subType'] = 'Elements/PageTS';
			}
			else {
				$fileData['subType'] = 'PageTS/Library';
			}
		}
		else if($relPathParts[2] === 'PageTS' && $fileData['extension']=='txt') {
			$fileData['type'] = 'TypoScript';
			$fileData['subType'] = 'PageTS';
		}
		
		// Constants & Setup
		else if($fileData['extension']=='constantsts') {
			$fileData['type'] = 'TypoScript';
			if($relPathParts[2] === 'Elements') {
				$fileData['subType'] = 'Elements/TypoScript';
			}
			else {
				$fileData['subType'] = 'TypoScript/Library';
			}
		}
		else if($fileData['extension']=='setupts') {
			$fileData['type'] = 'TypoScript';
			if($relPathParts[2] === 'Elements') {
				$fileData['subType'] = 'Elements/TypoScript';
			}
			else {
				$fileData['subType'] = 'TypoScript/Library';
			}
		}
		else if($fileData['name']=='setup.txt') {
			$fileData['type'] = 'TypoScript';
			if($relPathParts[3] === 'Extensions') {
				$fileData['subType'] = 'SetupTS';
			}
			else {
				$fileData['subType'] = 'TypoScript/SetupTS';
			}
		}
		else if($fileData['name']=='constants.txt') {
			$fileData['type'] = 'TypoScript';
			if($relPathParts[3] === 'Extensions') {
				$fileData['subType'] = 'ConstantsTS';
			}
			else {
				$fileData['subType'] = 'TypoScript/ConstantsTS';
			}
		}
		
		// Fluid
		else if($fileData['extension']=='html') {
			$fileData['type'] = 'Fluid';
			if($relPathParts[3] === 'Layouts' || $relPathParts[5] === 'Layouts') {
				$fileData['subType'] = $this->getSubType('Layouts', $relPathParts);
			}
			else if($relPathParts[3] === 'Partials' || $relPathParts[5] === 'Partials') {
				$fileData['subType'] = $this->getSubType('Partials', $relPathParts);
			}
			else if($relPathParts[3] === 'Templates' || $relPathParts[5] === 'Templates') {
				$fileData['subType'] = $this->getSubType('Templates', $relPathParts);
			}
			else if($relPathParts[5] === 'Widget' || $relPathParts[7] === 'Widget') {
				$fileData['subType'] = $this->getSubType('Widget', $relPathParts);
			}
		}
		
		// UserTS
		else 
		*/
		
		// Flexforms
		if($fileData['extension']=='xml' && $relPathParts[1] === 'Configuration') {
			$fileData['type'] = 'Flexforms';
			if($relPathParts[2] === 'Elements') {
				$fileData['subType'] = 'Elements/Flexforms';
			}
			else {
				$fileData['subType'] = '';
			}
		}
		
		else if(in_array($fileData['extension'], $languageTypes)) {
			//echo $relPathParts[3]."<br>";
			if($relPathParts[3] === 'Language' || $relPathParts[5] === 'Language') {
				$fileData['type'] = 'Language';
				$fileData['isEditable'] = TRUE;
				$fileData['subType'] = $fileData['extension']=='xml' ? 'XML' : 'XLIFF';
				$fileData['icon'] = 't3-icon t3-icon-flags t3-icon-flags-multiple t3-icon-multiple';
			}
		}

		else if(in_array($fileData['extension'], $mediaTypes)) {
			//echo $relPathParts[3]."<br>";
			//if($relPathParts[3] === 'Language' || $relPathParts[5] === 'Language') {
				$fileData['type'] = 'Media';
				$fileData['isEditable'] = TRUE;

				if(in_array($fileData['extension'], $fontTypes)) {
					$fileData['subType'] = 'Font';
				}
				if(in_array($fileData['extension'], $flashTypes)) {
					$fileData['subType'] = 'Flash';
				}
			
				$fileData['subType'] = $fileData['extension']=='xml' ? 'XML' : 'XLIFF';
				$fileData['icon'] = 't3-icon t3-icon-flags t3-icon-flags-multiple t3-icon-multiple';
			//}
		}

		// Files that are not writable, are not editable!
		if($fileData['isEditable']) {
			$fileData['isEditable'] = $fileData['isWritable'];
		}
		
		// Filter entries
		if(isset($this->filter[$fileData['type']])) {
			if($this->filter[$fileData['type']]==1) {
				$this->themeFileData[$relPathFilename] = $fileData;
			}
		}
		else {
			$fileData['type'] = 'X->' . $fileData['type'];
			$this->themeFileData[$relPathFilename] = $fileData;
		}

	}
	
	protected function getSubType($startFrom, $relPathParts) {
		$start = FALSE;
		$subTypeParts = array();
		foreach($relPathParts as $part) {
			if($part == $startFrom) {
				$start = TRUE;
			}
			if($start) {
				$subTypeParts[] = $part;
			}
		}
		return implode('/', $subTypeParts);
	}
	
}